﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;

namespace pac8017HWNet
{
    public class pac8017HW
    {
        private static int[] slotCurrentChIndex = new int[8];
        private static int[] slotCurrentGain = new int[8];
        private class ICPDAS
        {
            private const string lib = "pac_i8017HW_UWP_x64.dll";

            [DllImport(lib)]
            public static extern Int16 pac_i8017HW_GetLibVersion();

            [DllImport(lib)]
            public static extern void pac_i8017HW_GetLibDate(byte[] libDate);

            [DllImport(lib)]
            public static extern Int16 pac_i8017HW_GetFirmwareVersion(int slot, ref Int16 firmware);


            /// <summary>
            /// pac_i8017HW_Select_SingleEnd is only for 9017-15
            /// </summary>
            /// <param name="slot"></param>
            /// <param name="selection"></param>
            /// <returns></returns>
            [DllImport(lib)]
            public static extern Int16 pac_i8017HW_Select_SingleEnd(int slot, Int16 selection);

            [DllImport(lib)]
            public static extern Int16 pac_i8017HW_GetSingleEndJumper(int slot, ref Int16 Jumper);



            [DllImport(lib)]
            public static extern Int16 pac_i8017HW_ReadGainOffset_Info(int slot, int gain, ref UInt16 gainValue, ref Int16 offsetValue);

            [DllImport(lib)]
            public static extern Int16 pac_i8017HW_Read_mA_GainOffset(int slot, short ch, ref UInt16 GainValue, ref Int16 offsetValue);

            [DllImport(lib)]
            public static extern Int16 pac_i8017HW_Init(int slot);

            [DllImport(lib)]
            public static extern Int16 pac_i8017HW_SetLED(int slot, UInt32 led);

            [DllImport(lib)]
            public static extern Int16 pac_i8017HW_ReadAI(int slot, int ch, int gain, ref float fValue);


            [DllImport(lib)]
            public static extern Int16 pac_i8017HW_ReadAIHex(int slot, int ch, int gain, ref Int16 iValue);

            [DllImport(lib)]
            public static extern Int16 pac_i8017HW_ReadAI_AVG(int slot, int ch, int gain, UInt16 averageCnt, ref float fValue);


            [DllImport(lib)]
            public static extern Int16 pac_i8017HW_ReadAIHex_AVG(int slot, int ch, int gain, UInt16 averageCnt, ref Int16 iValue);


            [DllImport(lib)]
            public static extern Int16 pac_i8017HW_Get_D_Sub_Status(int iSlot, ref  Int16 D_Sub_Status);
        }

        /// <summary>
        /// Get the library version of this DLL (pac_i8017HW.dll)
        /// </summary>
        /// <returns>library version</returns>
        public static Int16 LibVersion()
        {
            return ICPDAS.pac_i8017HW_GetLibVersion();
        }

        /// <summary>
        /// Get the firmware version of the I-8017HW in the specified slot
        /// </summary>
        /// <param name="slot">1 ~ 7 (the slot which I-8017HW is inserted)</param>
        /// <param name="firmwareVersion">the read-back firmware version</param>
        /// <returns>the Error Codes which are defined in pac_i8017HW.h</returns>
        public static Int16 FirmwareVersion(int slot, ref Int16 firmware)
        {
            return ICPDAS.pac_i8017HW_GetFirmwareVersion(slot, ref firmware);
        }

        /// <summary>
        /// Get the building date of this DLL (pac_i8017HW.dll)
        /// </summary>
        /// <returns>library building date</returns>
        public static string LibDate()
        {
            //return Arr2Str(ICPDAS.pac_i8017H_GetLibDate());
            byte[] libDate = new byte[32];
            ICPDAS.pac_i8017HW_GetLibDate(libDate);
            //return libDate.Length.ToString();
            //Console.WriteLine("libDate= {0}  Len= {1}\n", AscArr2Str(libDate), libDate.Length.ToString());
            return AscArr2Str(libDate);
        }
        /// <summary>
        /// Initialize the I-8017HW in the specified slot
        /// </summary>
        /// <param name="slot">1 ~ 7 (the slot which I-8017HW is inserted)</param>
        /// <returns>the Error Codes which are defined in pac_i8017HW.h</returns>
        public static Int16 Init(int slot)
        {
            return ICPDAS.pac_i8017HW_Init(slot);
        }



        /// <summary>
        /// Read the analog input value in the floating-point format from the specified channel
        /// </summary>
        /// <param name="slot">1 ~ 7 (the slot which I-8017HW is inserted)</param>
        /// <param name="channel">0 ~ 7 if Differential mode; 0 ~ 15 if SingleEnded mode</param>
        /// <param name="gain">the gain which is defined by the enum Gain</param>
        /// <param name="fValue">the read back floating-point value</param>
        /// <returns>the Error Codes which are defined in pac_i8017HW.h</returns>
        public static Int16 ReadAI(int slot, int ch, int gain, ref float fValue)
        {

            return ICPDAS.pac_i8017HW_ReadAI(slot, ch, gain, ref fValue);
        }


        /// <summary>
        /// Read the analog input value in the hex format from the specified channel
        /// </summary>
        /// <param name="slot">1 ~ 7 (the slot which I-8017HW is inserted)</param>
        /// <param name="channel">0 ~ 7 if Differential mode; 0 ~ 15 if SingleEnded mode</param>
        /// <param name="gain">the gain which is defined by the enum Gain</param>
        /// <param name="hex16Bit">the read back value in hexadecimal format</param>
        /// <returns>the Error Codes which are defined in pac_i8017HW.h</returns>
        public static Int16 ReadAIHex(int slot, int ch, int gain, ref Int16 hex16Bit)
        {

            return ICPDAS.pac_i8017HW_ReadAIHex(slot, ch, gain, ref hex16Bit);
        }


        /// <summary>
        /// Read the analog input value in the floating-point format from the specified channel
        /// </summary>
        /// <param name="slot">1 ~ 7 (the slot which I-8017HW is inserted)</param>
        /// <param name="channel">0 ~ 7 if Differential mode; 0 ~ 15 if SingleEnded mode</param>
        /// <param name="gain">the gain which is defined by the enum Gain</param>
        /// <param name="averageCnt">the sample count to average the input values</param>
        /// <param name="fValue">the read back floating-point value</param>
        /// <returns>the Error Codes which are defined in pac_i8017HW.h</returns>
        public static Int16 ReadAI_AVG(int slot, int ch, int gain, UInt16 averageCnt, ref float fValue)
        {

            return ICPDAS.pac_i8017HW_ReadAI_AVG(slot, ch, gain, averageCnt, ref fValue);
        }


        /// <summary>
        /// Read the analog input value in the hex format from the specified channel
        /// </summary>
        /// <param name="slot">1 ~ 7 (the slot which I-8017HW is inserted)</param>
        /// <param name="channel">0 ~ 7 if Differential mode; 0 ~ 15 if SingleEnded mode</param>
        /// <param name="gain">the gain which is defined by the enum Gain</param>
        /// <param name="averageCnt">the sample count to average the input values</param>
        /// <param name="hex16Bit">the read back value in hexadecimal format</param>
        /// <returns>the Error Codes which are defined in pac_i8017HW.h</returns>
        public static Int16 ReadAIHex_AVG(int slot, int ch, int gain, UInt16 averageCnt, ref Int16 hex16Bit)
        {

            return ICPDAS.pac_i8017HW_ReadAIHex_AVG(slot, ch, gain, averageCnt, ref hex16Bit);
        }




        /// <summary>
        /// Get the type of the input channels; differential or single-ended
        /// </summary>
        /// <param name="slot">1 ~ 7 (the slot which I-8017HW is inserted)</param>
        /// <param name="selJumper">jumper status of input channel type; Differential or SingleEnded</param>
        /// <returns>the Error Codes which are defined in pac_i8017HW.h</returns>
        public static Int16 SingleEndJumper(int slot, ref Int16 selJumper)
        {
            return ICPDAS.pac_i8017HW_GetSingleEndJumper(slot, ref selJumper);
        }


        public static Int16 Select_SingleEnd_Differential(int slot, Int16 selection)
        {
            return ICPDAS.pac_i8017HW_Select_SingleEnd(slot, selection);
        }


        /// <summary>
        /// Get the gain value and the offset for a certain gain of I-8017HW in the specified slot
        /// </summary>
        /// <param name="slot">1 ~ 7 (the slot which I-8017HW is inserted)</param>
        /// <param name="gain">the gain which is defined by the enum Gain</param>
        /// <param name="gainValue">the calibated gain value</param>
        /// <param name="offset">the calibated offset value</param>
        /// <returns>the Error Codes which are defined in pac_i8017HW.h</returns>
        public static Int16 GainOffset_Info(int slot, int gain, ref UInt16 gainValue, ref Int16 offsetValue)
        {
            return ICPDAS.pac_i8017HW_ReadGainOffset_Info(slot, gain, ref gainValue, ref offsetValue);
        }


        public static Int16 Ch_mAGainOffset(int slot, int ch, ref UInt16 gainValue, ref Int16 offsetValue)
        {
            return ICPDAS.pac_i8017HW_Read_mA_GainOffset(slot, (Int16)ch, ref gainValue, ref offsetValue);
        }


        /// <summary>
        /// Set the LEDs on the I-8017HW in the specified slot
        /// </summary>
        /// <param name="slot">1 ~ 7 (the slot which I-8017HW is inserted)</param>
        /// <param name="led">0 ~ 0xFFFF, 16-bit unsigned value which represents the 16 LEDs</param>
        /// <returns>the Error Codes which are defined in pac_i8017HW.h</returns>
        public static Int16 SetLED(int slot, UInt32 led)
        {
            return ICPDAS.pac_i8017HW_SetLED(slot, led);
        }


        public static Int16 D_Sub_Status(int slot, ref Int16 status)
        {
            return ICPDAS.pac_i8017HW_Get_D_Sub_Status(slot, ref status);
        }

        private static string AscArr2Str(byte[] b)
        {
            return System.Text.UnicodeEncoding.Unicode.GetString(
            System.Text.ASCIIEncoding.Convert(System.Text.Encoding.ASCII,
            System.Text.Encoding.Unicode, b), 0, b.Length);
        }
        public static string Arr2Str(byte[] buffer)
        {
            return (new UnicodeEncoding()).GetString(buffer, 0, buffer.Length);
        }


    }
}

